/*
NAME: ThunderBrowse AutoComplete Component.
AUTH: GameSpotting! Networks
DESC: Searches your history for relative urls.

The following code contains portions of/based on component code from
  the QuickFile extension (licensed under MPL 1.1) by Paul Tomlin and Mark Shuttleworth
     used in complience with the license
  Mozilla Developer Documentation  
*/
const Ci = Components.interfaces;

const CLASS_ID = Components.ID("{4d85a9bc-23a2-11dc-8314-0800200c9a66}");
const XPFE_CLASS_ID = Components.ID("{53D47A95-4F63-4438-9CF0-9EC094412142}");
const CLASS_NAME = "ThunderBrowse History";
const XPFE_CLASS_NAME = "ThunderBrowse History";         
const CONTRACT_ID = "@mozilla.org/autocomplete/search;1?name=tbhistory";
const XPFE_CONTRACT_ID = "@mozilla.org/autocompleteSession;1?type=tbhistory";

// Implements nsIAutoCompleteResult
function ThunderBrowseCompleteResult(searchString, searchResult,
                                  defaultIndex, errorDescription,
                                  results, comments) {
  this._results = null
  this._searchResult = null;                                
  this._searchString = searchString;
  this._searchResult = searchResult;
  this._defaultIndex = defaultIndex;
  this._errorDescription = errorDescription;
  this._results = results;
  this._comments = comments;
}
ThunderBrowseCompleteResult.prototype = {
  _searchString: "",
  _searchResult: 0,
  _defaultIndex: 0,
  _errorDescription: "",
  _results: [],
  _comments: [],

  get searchString() {return this._searchString;},
  get searchResult() {return this._searchResult;},
  get defaultIndex() {return this._defaultIndex;},
  get errorDescription() {return this._errorDescription;},
  get matchCount() {return this._results.length;},
  getValueAt: function(index) {return this._results[index];},
  getCommentAt: function(index) {return this._comments[index];},
  getLabelAt: function(index) { return this._results[index]; },
  getStyleAt: function(index) {
    if (!this._comments[index])
      return null;  // not a category label, so no special styling
    if (index == 0)
      return "suggestfirst";  // category label on first line of results

    return "suggesthint";   // category label on any other line of results
  },
  getImageAt : function (index) {return "";},
  removeValueAt: function(index, removeFromDb) {
    this._results.splice(index, 1);
    this._comments.splice(index, 1);
  },
  QueryInterface: function(aIID) {
    if (!aIID.equals(Ci.nsIAutoCompleteResult) && !aIID.equals(Ci.nsIAutoCompleteResult))
        throw Components.results.NS_ERROR_NO_INTERFACE;
    return this;
  }
};
function ThunderBrowseAutoComplete () 
{
  this._results = null;
  this._observer = null;
}
ThunderBrowseAutoComplete.prototype = {

  decodeUri: function(url)
  {
    if(url.split("ENC:")[1] != null)
    {
     var removePrefix = url.split("ENC:")[1];
     var decodeURI = decodeURIComponent(removePrefix);
     return decodeURI;
    }
    else
     return url;
  },
  dupeRemov: function(a)
  {
   var r = new Array();
   o:for(var i = 0, n = a.length; i < n; i++)
   {
      for(var x = 0, y = r.length; x < y; x++)
      {
         if(r[x]==a[i]) continue o;
      }
      r[r.length] = a[i];
   }
   return r;
  },
  _initXPFEResults: function __initXPFEResults ()
  {
    this._results = Components.classes['@mozilla.org/autocomplete/results;1']
                              .createInstance(Ci.nsIAutoCompleteResults);
  },
  _appendXPFEResult: function __appendXPFEResult (aFullName)
  {
    var result = Components.classes['@mozilla.org/autocomplete/item;1']
                           .createInstance(Ci.nsIAutoCompleteItem);
    result.value = aFullName;
    result.comment = null;

    this._results.items.AppendElement(result);
  },
  startSearch: function(searchString, searchParam, result, listener)
  {
    this.search(searchString, searchParam, result, listener);
  },  
  onStartLookup: function _onStartLookup (aSearchString, aPreviousSearchResult, aListener)
  {

    if (this._observer != aListener)
      this._observer = aListener;

    this._initXPFEResults();
    this._results.searchString = aSearchString;

    this.search(aSearchString, aPreviousSearchResult, null, aListener);
  },
  search: function(searchString, searchParam, result, listener) {

    var prefs = Components.classes["@mozilla.org/preferences-service;1"].getService(Ci.nsIPrefService)
    var listofUrls = prefs.getBranch("extensions.tbrowse.").getCharPref("dropdown");
    if (listofUrls.length > 0) 
    {
      var results = null;
      var compvistedURLs = listofUrls.split(",");
      var results = [];
      var comments = [];
      var vistedURLs = this.dupeRemov(compvistedURLs);
      for (var i=0; i<vistedURLs.length; i++) 
      {
	var decodedurl = this.decodeUri(vistedURLs[i]);
        if(decodedurl.toLowerCase().match(searchString.toLowerCase()) != (null && ""))
	{
	   if(typeof listener.onSearchResult == 'undefined')
	    this._appendXPFEResult(decodedurl);
	   else
	   {
	    results.push(decodedurl);
	    //comments.push(decodedurl); 
           }
        }	
      }
      if(typeof Ci.nsIAutoCompleteResult != 'undefined')
      {
       var newResult = new ThunderBrowseCompleteResult(searchString, Ci.nsIAutoCompleteResult.RESULT_SUCCESS, 
	     	      	  					    0, "", results, comments);
       listener.onSearchResult(this, newResult);
      }
      else
      {
       var status = Ci.nsIAutoCompleteStatus.noMatch;
       if (this._results.items.Count())
        status = Ci.nsIAutoCompleteStatus.matchFound; 
       listener.onAutoComplete(this._results, status);
      }
    }
  },
  stopSearch: function() { /* null */ }, 
  onStopLookup: function _onStopLookup (){/* nada */},
  onAutoComplete: function _onAutoComplete (aSearchString, aPreviousSearchResult, aListener){/* nada */}, 
  QueryInterface: function(aIID) 
  {
    if (!aIID.equals(Ci.nsIAutoCompleteSearch) 
     && !aIID.equals(Ci.nsISupports) 
     && !aIID.equals(Ci.nsIAutoCompleteSession))
        throw Components.results.NS_ERROR_NO_INTERFACE;
    return this;
  }
};

// Factory
var ThunderBrowseCompleteSearchFactory = {
  createInstance: function (aOuter, aIID) {
    if (aOuter != null)
      throw Components.results.NS_ERROR_NO_AGGREGATION;
    return (new ThunderBrowseAutoComplete()).QueryInterface(aIID);
  }
};
// Module
var ThunderBrowseCompleteSearchModule = {
  registerSelf: function(aCompMgr, aFileSpec, aLocation, aType) {
    aCompMgr = aCompMgr.QueryInterface(Components.interfaces.nsIComponentRegistrar);
    if(typeof Ci.nsIAutoCompleteResult == 'undefined')
     aCompMgr.registerFactoryLocation(XPFE_CLASS_ID, XPFE_CLASS_NAME, XPFE_CONTRACT_ID,aFileSpec, aLocation, aType);
    else                                 
     aCompMgr.registerFactoryLocation(CLASS_ID, CLASS_NAME, CONTRACT_ID, aFileSpec, aLocation, aType);

  },
  unregisterSelf: function(aCompMgr, aLocation, aType) {
    aCompMgr = aCompMgr.QueryInterface(Ci.nsIComponentRegistrar);
    if(typeof Ci.nsIAutoCompleteResult == 'undefined')
     aCompMgr.unregisterFactoryLocation(XPFE_CLASS_ID, aLocation); 
    else
     aCompMgr.unregisterFactoryLocation(CLASS_ID, aLocation);   
  },
  
  getClassObject: function(aCompMgr, aCID, aIID) {
    if (!aIID.equals(Components.interfaces.nsIFactory))
      throw Components.results.NS_ERROR_NOT_IMPLEMENTED;

    if (aCID.equals(CLASS_ID) || aCID.equals(XPFE_CLASS_ID))
      return ThunderBrowseCompleteSearchFactory;

    throw Components.results.NS_ERROR_NO_INTERFACE;
  },

  canUnload: function(aCompMgr) { return true; }
};

// Module initialization
function NSGetModule(aCompMgr, aFileSpec) { return ThunderBrowseCompleteSearchModule; }

function NSGetFactory(cid) {
  if (cid.toString().toUpperCase() != CLASS_ID.toString().toUpperCase()) {
    throw Components.results.NS_ERROR_FACTORY_NOT_REGISTERED;
  }
  return ThunderBrowseCompleteSearchFactory;

}